import thorpy
import game
from ia import ImprovedRandomSolver as Solver

def get_ia(board, name):
    if name == "Beginner":
        return Solver(board, 50)
    elif name == "Normal":
        return Solver(board, 150)
    elif name == "Hard":
        return Solver(board, 300)
    else:
        return None

def build_player_option(name):
    varset = thorpy.VarSet()
    varset.add("name", name, "Name: ")
    varset.add("type", ["Human","Beginner", "Normal", "Hard"],
                            "Type: ")
    color=(0,255,0) if name=="Player 1" else (0,0,255) #color is tunable by user
    varset.add("color", color, "Color: ")
    e = thorpy.ParamSetterLauncher.make([varset], name, name+" options")
    return varset, e

def set_game_and_play():
    n = varset.get_value("n")
    columns = varset.get_value("columns")
    lines = varset.get_value("lines")
    #some verification
    if max((columns,lines)) < n:
        thorpy.launch_blocking_alert(
                            "The board is not large enough compared to N (N="+\
                            str(n)+").",
                            parent=e_background,
                            font_size=thorpy.style.FONT_SIZE+3,
                            font_color=(255,0,0))
    max_w = 0.75*thorpy.get_screen().get_width()
    max_h = 0.75*thorpy.get_screen().get_height()
##    d/gap = 15 and w,h = (d+gap)*nx, (d+gap)*ny
##    w = (d+d/15)*nx < max_w ==> d < max_w / (nx*(1+1/15))
    d1 = max_w / (columns*(1.+1./15))
    d2 = max_h / (lines*(1.+1./15))
    d = int(min(d1, d2))
    c1, c2 = varp1.get_value("color"), varp2.get_value("color")
    p1name, p2name = varp1.get_value("name"), varp2.get_value("name")
    g = game.Game(n, columns, lines, d, d//15, p1name, p2name, c1, c2)
    g.ia[1] = get_ia(g.board, varp1.get_value("type"))
    g.ia[-1] = get_ia(g.board, varp2.get_value("type"))
    m_play = thorpy.Menu(g.e_bck)
    m_play.play()
    e_background.blit()
    e_background.update()

def launch_credits():
    thorpy.launch_alert("This program was written in Python 3 by Yann"+\
                        "Thorimbert\nThe library Thorpy (www.thorpy.org) "+\
                        "written for pygame was used.")


METADATA_PATH = "./metadata" #create a file for storing data on application
mdm = thorpy.MetaDataManager()
mdm.read_data(METADATA_PATH)
#try loading screen size and flags (with default values indicated as arguments)
w,h,flags = mdm.get_display_data(METADATA_PATH, w=800, h=600, flags=0)
app = thorpy.Application((w,h), "NinArow", flags=flags)

thorpy.set_theme("human")
mdm.load_font_data(METADATA_PATH)

e_play = thorpy.make_button("Play", set_game_and_play)

varset = thorpy.VarSet() #here we will declare options that user can set
varset.add("n", value=4, text="N:", limits=(2, 20))
varset.add("columns", value=7, text="Columns:", limits=(2, 20))
varset.add("lines", value=6, text="Lines:", limits=(2, 20))

e_options = thorpy.ParamSetterLauncher.make([varset], "Game Options", "Options")

varp1, e_p1 = build_player_option("Player 1")
varp2, e_p2 = build_player_option("Player 2")
e_player_options = thorpy.make_group([e_p1,e_p2])

e_thorpy_options = thorpy.make_global_display_options(METADATA_PATH,
                                                        "Application options")

e_credits= thorpy.make_button("Credits", launch_credits)

e_quit = thorpy.make_button("Quit", thorpy.functions.quit_menu_func)

elements = [e_play,e_options,e_player_options,e_thorpy_options,e_credits,e_quit]
e_background = thorpy.Background.make(elements=elements)
thorpy.store(e_background)

m_main = thorpy.Menu(e_background)
m_main.play()

app.quit()


